/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  http://www.obm.org/
 *
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

var activeCalendarManager = null;
function getCalendarManager() {
	if (!activeCalendarManager) {
		activeCalendarManager = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
	}
	return activeCalendarManager;
}

function calOBMCalendar() {
	this.wrappedJSObject = this;
	// a reference to the underlying native calendar storage
	this.mLocalCalendar = null;
	// a reference to our calOBMObserver, to know if it has been added or not
	// into the mLocalCalendar object
	this.mObserver = null;
	try {
		//Init logger
		this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
		var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
		this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
		this.logger.setHeader("calOBMCalendar: ");
		this.logger.debug("new calOBMCalendar()", false, false);

		var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader);
		if (typeof(UtilsOBM) == "undefined") {
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/UtilsOBM.js");
		}

		if (typeof(Calendar_Event_Synchronizer) == "undefined") {
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/synchronizer/Calendar_Event_Synchronizer.js");
		}
		this.mEventSynchronizer = new Calendar_Event_Synchronizer();

		if (typeof(Calendar_Task_Synchronizer) == "undefined") {
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/synchronizer/Calendar_Task_Synchronizer.js");
		}
		this.mTaskSynchronizer = new Calendar_Task_Synchronizer();

	} catch(e) {
		this.logger.error(e, true, false);
	}

	try {
		if ( typeof(Calendar_Email_Notification_Dialog) == "undefined") {
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/Calendar_Email_Notification_Dialog.js");
		}
	} catch(e) {
		this.logger.debug("MBA: explosion");
		this.logger.error(e, true, false);
	}

}

calOBMCalendar.prototype = {
	//
	// nsISupports interface
	//
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports)
				&& !aIID.equals(Ci.calICalendar)
				&& !aIID.equals(Ci.calICalendarProvider)
				&& !aIID.equals(Ci.nsIRunnable)
				&& !aIID.equals(Ci.calISchedulingSupport)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	//
	// nsIRunnable
	//
	run: function() {
		this.refresh();
	},

	//
	// calICalendar interface
	//
	mID: null,
	get id() {
		return this.mID;
	},
	set id(id) {
		if (this.mID) {
			throw Cr.NS_ERROR_ALREADY_INITIALIZED;
		}
		this.mLocalCalendar.id = id;
		dump("this.mLocalCalendar.id:" + this.mLocalCalendar.id + "\n");
		return (this.mID = id);
	},

	// attribute AUTF8String name;
	get name() {
		var name = getCalendarManager().getCalendarPref_(this, "NAME");
		if (!this.mLocalCalendar.name) {
			//mLocalCalendar.name is needed for light 1.0
			this.mLocalCalendar.name = name;
		}
		return name;
	},
	set name(name) {
		getCalendarManager().setCalendarPref_(this, "NAME", name);
		return (this.mLocalCalendar.name = name);
	},

	// readonly attribute AUTF8String type;
	get type() {
		return "obm";
	},

	// FIXME : needed for light 0.9
	get superCalendar() {
		// If we have a superCalendar, check this calendar for a superCalendar.
		// This will make sure the topmost calendar is returned
		return (this.mSuperCalendar ? this.mSuperCalendar.superCalendar: this);
	},
	set superCalendar(val) {
		return (this.mSuperCalendar = val);
	},

	// attribute nsIURI uri;
	mUri: null,
	get uri() {
		try {
			return this.mURI;
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},
	set uri(aURI) {
		try {
			if (!this.mURI) {
				dump("set uri" + aURI.spec + "\n");
				this.mURI = aURI;
				this.mOBMName = this.mURI.spec.split(";").pop();
				this.initLocalCalendar(aURI);
			}
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// attribute boolean readOnly;
	mReadOnly: null,
	get readOnly() {
		try {
			//this.logger.debug("get readOnly", false, false);
			if (this.synchronizing) {
				//this.logger.debug(this.mOBMName + "-> synchronizing", false, false);
				return false;
			} else {
				// can't use calendarManager.get/setCalendarPref_() here ! bug #313948
				// have to resort using prefs.js...
				//this.logger.debug(this.mOBMName + "->"+(UtilsOBM.getBoolPref("extensions.obm.changes.calICalendar."
				//+ this.mOBMName + ".readOnly", false)), false, false);
				return UtilsOBM.getBoolPref("extensions.obm.changes.calICalendar." + this.mOBMName + ".readOnly", false);
			}
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},
	set readOnly(bool) {
		// nothing, user must not be able to change it
		this.logger.debug("set readOnly (" + bool + ")", false, false);
	},

	// readonly attribute boolean canRefresh;
	get canRefresh() {
		return true;
	},

	// nsIVariant getProperty(in AUTF8String aName);
	getProperty: function(aName) {
		//this.logger.debug("getProperty("+aName+")", false, false);
		if (aName.match("identity")) {
			return this.mLocalCalendar.getProperty(aName);
		} else {
			return this.mLocalCalendar.getProperty(aName);
		}
	},
	// void setProperty(in AUTF8String aName, in nsIVariant aValue);
	setProperty: function(aName, aValue) {
		//this.logger.debug("setProperty("+aName+","+aValue+")", false, false);
		this.mLocalCalendar.setProperty(aName, aValue);
	},
	// void deleteProperty(in AUTF8String aName);
	deleteProperty: function(aName) {
		var calMgr = getCalendarManager();
		calMgr.deleteCalendarPref_(this, aName);
	},
	// void addObserver( in calIObserver observer )
	addObserver: function(aObserver) {
		try {
			if (!this.mObserver) { // to be sure to have our observer. Not sure it is necessary
				this.mObserver = new calOBMObserver(this, this.mOBMName, this.mEventSynchronizer, this.mTaskSynchronizer);
				this.mLocalCalendar.addObserver(this.mObserver);
			}
			// and add lightning observer
			this.mLocalCalendar.addObserver(aObserver);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void removeObserver( in calIObserver observer );
	removeObserver: function(aObserver) {
		try {
			this.mLocalCalendar.removeObserver(aObserver);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void addItem( in calIItemBase aItem, in calIOperationListener aListener );
	addItem: function(aItem, aListener) {
		this.logger.debug("addItem", false, false);
		var isMutable = aItem.isMutable;
		try {
			if (this.readOnly) {
				throw Ci.calIErrors.CAL_IS_READONLY;
			}

			var addedThroughICS = false;
			// check if the event is added through the email blue bar
			if (aItem instanceof Ci.calIEvent
				&& ( aItem.hasProperty("X-MOZ-RECEIVED-SEQUENCE") || aItem.hasProperty("X-MOZ-RECEIVED-DTSTAMP") ) ) {
				addedThroughICS = true;
			}
			
			if ( ! isMutable ) {
				aItem = aItem.clone();
			}

			// the event has been added using Lightning GUI : 
			// we set the X-OBMEXT-IS-INTERNAL flag
			if (aItem instanceof Ci.calIEvent && !addedThroughICS ) {
				aItem.setProperty('X-OBMEXT-IS-INTERNAL', 'YES');
			}

			if (aItem instanceof Ci.calIEvent
				&& aItem.hasProperty('X-OBM-DOMAIN')) {

				// let's check if this is an internal OBM event
				// if this is it, this is probably an event added when
				// the user "accept" or "decline" a participation from
				// the mail sent for the participation
				// note: we cannot do this directly into the observer, into onAddedItem
				// because onAddedItem is called when adoptItem is called
				// by our synchronizer during the import of new events
				var obmDomain =  aItem.getProperty('X-OBM-DOMAIN');
				var authDomain = UtilsOBM.getCharPref("extensions.obm.server.domain", "");

				if (authDomain != '' && authDomain == obmDomain) {
					// this is an internal event, already created into our server
					// we should then notify as modified, not as added.
					// we mark our event. the property will be removed later
					aItem.setProperty('X-OBMEXT-PARTICIPATION-CHANGED', 'YES');
					aItem.setProperty('X-OBMEXT-IS-INTERNAL', 'YES');
					this.logger.debug(" set property X-OBMEXT-PARTICIPATION-CHANGED", false, false);
				}
			}
			/*
			// try to add organizer to attendees of the event
			if ( aItem.organizer && aItem.organizer.id ) {
				var organizerFound = false;
				var atts = aItem.getAttendees({});
				for ( var i in atts ) {
					this.logger.debug("attendees...",false,false);
					this.logger.debug(atts[i].id,false,false);
					this.logger.debug(atts[i].commonName,false,false);
					if ( atts[i].id == aItem.organizer.id ) {
						organizerFound = true;
					}
				}
				// add organizer to the attendees
				if ( !organizerFound ) {
					aItem = aItem.clone();
					var att = aItem.organizer.clone();
					att.isOrganizer = true;
					aItem.addAttendee(att);
				}
				this.logger.debug("organizer...",false,false);
				this.logger.debug(aItem.organizer.id,false,false);
				this.logger.debug(aItem.organizer.commonName,false,false);
			}
			*/
			if ( ! isMutable ) {
				aItem.makeImmutable();
			}
			this.mLocalCalendar.addItem(aItem, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void adoptItem( in calIItemBase aItem, in calIOperationListener aListener );
	adoptItem: function(aItem, aListener) {
		try {
			this.logger.debug("adoptItem", false, false);
			if (this.readOnly) {
				throw Ci.calIErrors.CAL_IS_READONLY;
			}
			this.mLocalCalendar.adoptItem(aItem, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	_canModifyItem: function(aNewItem, aOldItem, callback) {
		if (!aOldItem) {
			// no old item found?  should be using addItem, then.
			// let calendar report the error
			return callback(false);
		}
		var listener = {
			onOperationComplete: function(aCalendar, aStatus, aOperationType, aId, aDetail) {
				if (!this.storedOldItem) {
					// no old item found in calendar ?  should be using addItem, then.
					// let calendar report the error
					return callback(false);
				}

				aOldItem = aOldItem.parentItem;

				if (aOldItem.generation != this.storedOldItem.generation) {
					return callback(false);
				}
				return callback(true);
			},
			onGetResult: function(aCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
				if (aCount > 0) {
					this.storedOldItem = aItems[0];
				}
			}
		};
		listener.storedOldItem = null;
		this.mLocalCalendar.getItem(aOldItem.id, listener);
	},

	// void modifyItem( in calIItemBase aNewItem, in calIItemBase aOldItem,
	//                  in calIOperationListener aListener );
	modifyItem: function(aNewItem, aOldItem, aListener) {
		this.logger.debug("modifyItem (" + this.mOBMName + ")", false, false);
		try {
			if (this.readOnly) {
				//this.logger.debug("modifyItem: error, readonly", false, false);
				throw Ci.calIErrors.CAL_IS_READONLY;
			}
			var self = this;
			this._canModifyItem(aNewItem, aOldItem, function (canModify) {
				//self.logger.debug("callback of _canModifyItem", false, false);
				if (!canModify) {
					var stringBundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
						.getService(Components.interfaces.nsIStringBundleService)
						.createBundle("chrome://obm-extension/locale/calendar/errors_warnings.properties");
					var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
						.getService(Components.interfaces.nsIPromptService);
					promptService.alert(null,stringBundle.GetStringFromName("obm.modify.alert.title"),stringBundle.GetStringFromName("obm.modify.alert.content"));
				} else {
					try {
						//var diffLevel = UtilsOBM.diffIsOnlyParticipation(aNewItem, aOldItem);
						//if (diffLevel == 1) {
							//aNewItem.generation = aOldItem.generation;
						//}

						self.mLocalCalendar.modifyItem(aNewItem, aOldItem, aListener);
						//self.logger.debug("modifyItem on mLocalCalendar ok ", false, false);
					} catch(e) {
						//self.logger.debug("modifyItem on mLocalCalendar: error ", false, false);
						self.logger.error(e, true, false);
					}
				}
			});
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void deleteItem( in calIItemBase aItem, in calIOperationListener aListener );
	deleteItem: function(aItem, aListener) {
		this.logger.debug("deleteItem (" + this.mOBMName + ")", false, false);
		if (this.readOnly) {
			throw Ci.calIErrors.CAL_IS_READONLY;
		}
		this.mLocalCalendar.deleteItem(aItem, aListener);
	},

	// void getItem( in string aId, in calIOperationListener aListener );
	getItem: function(aId, aListener) {
		try {
			this.logger.debug("getItem (" + this.mOBMName + ")", false, false);
			return this.mLocalCalendar.getItem(aId, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void getItems( in unsigned long aItemFilter, in unsigned long aCount,
	//                in calIDateTime aRangeStart, in calIDateTime aRangeEndEx,
	//                in calIOperationListener aListener );
	getItems: function(aItemFilter, aCount, aRangeStart, aRangeEnd, aListener) {
		try {
// 			this.logger.debug("getItems (" + this.mOBMName + ")", false, false);
			return this.mLocalCalendar.getItems(aItemFilter, aCount, aRangeStart, aRangeEnd, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// attribute boolean suppressAlarms;
	get suppressAlarms() {
		return false;
	},
	set suppressAlarms(aSuppressAlarms) {
		throw Cr.NS_ERROR_NOT_IMPLEMENTED;
	},

	// readonly attribute boolean sendItipInvitations;
	get sendItipInvitations() {
		return false;
	},

	// void refresh();
	refresh: function() {
		if ( !UtilsOBM.isOnline() ) {
			this.logger.debug("refresh canceled: we're offline", false, false);
			return ;
		}
		this.logger.debug("try to refresh()", false, false);
		if (UtilsOBM.getBoolPref("extensions.obm.changes.calICalendar." + this.mOBMName + ".firstRefresh", true)) {
			this.logger.debug("refresh canceled", false, false);
			UtilsOBM.setBoolPref("extensions.obm.changes.calICalendar." + this.mOBMName + ".firstRefresh", false);
			return;
		}
		var wWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].getService(Components.interfaces.nsIWindowWatcher);
		var window = wWatcher.getWindowByName("OBM_SYNCHRO", null);
		if (window != null) {
			this.logger.debug("refresh canceled : sync window open", false, false);
			return;
		}

		var we = wWatcher.getWindowEnumerator();
		while ( we.hasMoreElements() ) {
		  var testw = we.getNext();
		  this.logger.debug("Window name:"+testw.name,false,false);
		  if ( testw.name.indexOf("notificationDialog:") == 0 ) {
			// there is at least one notification dialog window opended: we cancel the sync
			this.logger.debug("refresh canceled : notification window open", false, false);
			return ;
		  }
		}


		var syncManager = Components.classes["@obm.org/syncmanager;1"].createInstance(Components.interfaces.obmISyncManager);
		syncManager.syncCalendar(this);
	},

	// void startBatch();
	startBatch: function() {
		//this.logger.debug(this.mOBMName +" -> start batch readOnly:" + this.mLocalCalendar.readOnly, false, false);
		this.synchronizing = true;
		//this.mLocalCalendar.startBatch();
		this.mReadOnlyStateBeforeBatch = this.mLocalCalendar.readOnly;
		this.mLocalCalendar.removeObserver(this.mObserver);
		this.mLocalCalendar.readOnly = false;
	},

	// void endBatch();
	endBatch: function() {
		if (this.synchronizing === true) {
			this.synchronizing = false;
			this.mLocalCalendar.readOnly = this.mReadOnlyStateBeforeBatch;
			//this.mLocalCalendar.endBatch();
			this.mLocalCalendar.addObserver(this.mObserver);
		}
		//this.logger.debug(this.mOBMName +" -> end batch readOnly:" + this.mLocalCalendar.readOnly, false, false);
	},

	//
	// calICalendarProvider interface
	//
	get prefChromeOverlay() {
		return null;
	},

	get displayName() {
		return "Remote OBM Calendar";
	},

	createCalendar: function() {
		throw NS_ERROR_NOT_IMPLEMENTED;
	},

	deleteCalendar: function(aCalendar, aListener) {
		try {
			this.mLocalCalendar.QueryInterface(Ci.calICalendarProvider).deleteCalendar(this.mLocalCalendar, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	//
	// calISchedulingSupport
	//

	isInvitation: function(aItem) {
		return this.mLocalCalendar.superCalendar.QueryInterface(Ci.calISchedulingSupport).isInvitation(aItem);
	},

	getInvitedAttendee: function(aItem) {
		return this.mLocalCalendar.superCalendar.QueryInterface(Ci.calISchedulingSupport).getInvitedAttendee(aItem);
	},

	canNotify: function(aMethod, aItem) {
		return this.mLocalCalendar.superCalendar.QueryInterface(Ci.calISchedulingSupport).canNotify(aMethod, aItem);
	},

	//
	// PRIVATE METHODS
	//
	initLocalCalendar: function(aUri) {
		try {
			this.logger.debug("initLocalCalendar()", false, false);
			this.logger.debug("aUri: "+aUri, false, false);
			this.mLocalCalendar = Cc["@mozilla.org/calendar/calendar;1?type=storage"].createInstance(Ci.calICalendar);
			this.mLocalCalendar.uri = aUri;
			// NOT supported in "storage" calendar, waiting for bug #341776
			// FIXME : fixed in light 0.9
			//this.mLocalCalendar.wrappedJSObject.superCalendar = this;
			this.mObserver = new calOBMObserver(this, this.mOBMName, this.mEventSynchronizer, this.mTaskSynchronizer);
			this.mLocalCalendar.addObserver(this.mObserver);

			this.mLocalCalendar.readOnly = UtilsOBM.getBoolPref("extensions.obm.changes.calICalendar." + this.mOBMName + ".readOnly", false);

		} catch(e) {
			this.logger.error(e, true, false);
		}
	}

};

function calOBMObserver(aOBMCalendar, aOBMName, aEventSynchronizer, aTaskSynchronizer) {
	//Init logger
	this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
	var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
	this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
	this.logger.setHeader("calOBMObserver: ");
	this.logger.debug("calOBMObserver()", false, false);

	this.mOBMCalendar = aOBMCalendar;
	this.mOBMName = aOBMName;
	this.mEventSynchronizer = aEventSynchronizer;
	this.mTaskSynchronizer = aTaskSynchronizer;
}

calOBMObserver.prototype = {

	//
	// calIObserver interface
	//
	onStartBatch: function() {
		this.logger.debug("onStartBatch", false, false);
	},

	onEndBatch: function() {
		this.logger.debug("onEndBatch", false, false);
	},

	onLoad: function() {
		this.logger.debug("onLoad", false, false);
	},

	onAddItem: function(aAddedItem) {
		this.logger.debug("onAddItem", false, false);

		if (aAddedItem instanceof Ci.calIEvent) {
			// let's check if this is a new internal OBM event on which
			// only participation has been changed when
			// the user "accept" or "decline" a participation from
			// the mail sent for the participation
			// for this, we check the flag that it indicates we are not
			// in the case were the event is adopted during the synchronisation
			if (aAddedItem.hasProperty('X-OBMEXT-PARTICIPATION-CHANGED')) {
				this.logger.debug("  the added event should be only modified as participation changed on remote server", false, false);
				this.mEventSynchronizer.notifyItemParticipationChanged(this.mOBMCalendar, aAddedItem);
				var notif = new Calendar_Email_Notification_Dialog();
				notif.handleNotification(aAddedItem,"added");
			}
			else {
				var notif = new Calendar_Email_Notification_Dialog();
				notif.handleNotification(aAddedItem,"added");

				this.mEventSynchronizer.notifyItemAdded(this.mOBMCalendar, aAddedItem);
			}
			this.logger.debug("aItem id: "+aAddedItem.id,false,false);
			this.logger.debug("aItem uid: "+aAddedItem.uid,false,false);
			this.logger.debug("aItem extId: "+aAddedItem.extId,false,false);
		} else if (aAddedItem instanceof Ci.calITodo) {
			this.mTaskSynchronizer.notifyItemAdded(this.mOBMCalendar, aAddedItem);
		}
	},

	onModifyItem: function(aNewItem, aOldItem) {
		this.logger.debug("onModifyItem(" + aNewItem.title + " <- " + aOldItem.title + ")", false, false);
// 		this.logger.debug(aNewItem.icalString, false, false);
		this.logger.debug("oldItem generation: "+aOldItem.generation, false, false);
		this.logger.debug("newItem generation: "+aNewItem.generation, false, false);
		if (aNewItem instanceof Ci.calIEvent) {
			this.logger.debug("onModifyItem: this item is an event", false, false);
			this.logger.debug("aItem id: "+aNewItem.id,false,false);
			this.logger.debug("aItem uid: "+aNewItem.uid,false,false);
			this.logger.debug("aItem extId: "+aNewItem.extId,false,false);
			var diffLevel ;
			// 2 cases : the user is the organizer, or the user is an attendee
			// a/ usera invites userb
			// b/ userb accepts
			// c/ usera change the date of the event
			// d/ userb get the email notification but has not synchronized
			//    => we should send to obm-sync a participationChange although the date changed
			var ownerName = UtilsOBM.getCalendarManager().getCalendarPref_(aOldItem.calendar, "X-OBM-EMAIL");
// 			this.logger.debug("ownerName: "+ownerName,false,false);
			
			if (!aNewItem.hasProperty('X-OBM-DOMAIN') && !aNewItem.hasProperty('X-OBMEXT-IS-INTERNAL') ) {
				//if the event is an external it's sure we should send to the server an update request
// 				this.logger.debug("aNewItem doesn't have the X-OBM-DOMAIN nor the X-OBMEXT-IS-INTERNAL property: it's not an external event", false, false);
				this.logger.debug("onModifyItem: this event is an external event", false, false);
				diffLevel = UtilsOBM.diffOfAttendee(aNewItem,aOldItem);
				if ( diffLevel ) {
					var notif = new Calendar_Email_Notification_Dialog();
					notif.handleNotification(aNewItem,"updated",aOldItem);
					this.mEventSynchronizer.notifyItemModified(this.mOBMCalendar, aNewItem);
				}
				return ;
			}
				
			try {
				var eventOrganizer = aOldItem.organizer;
				var isOrganizer;
				if (!eventOrganizer) {
					// Not sure this is possible but you never know
					// Set isOrganizer to true in order to force a full
					// difference of fields
					this.logger.warning("Woops! No organizer for this event");
					isOrganizer = true;
				}
				else
					isOrganizer = (eventOrganizer.id == "mailto:" + ownerName);
				if ( isOrganizer ) {
					this.logger.debug("The owner of the calendar is the organizer, checking for differences in all fields");
					diffLevel = UtilsOBM.diffIsOnlyParticipation(aNewItem, aOldItem);
				} else {
					this.logger.debug("The owner of the calendar is not the organizer, checking for differences in the attendees");
					diffLevel = UtilsOBM.diffOfAttendee(aNewItem,aOldItem);
				}
			} catch(e) {
				diffLevel = UtilsOBM.diffIsOnlyParticipation(aNewItem, aOldItem);
			}

			
			
			if (diffLevel == 1) {
				this.logger.debug("onModifyItem: the only modification made on it is a 'participation state change'", false, false);
				var notif = new Calendar_Email_Notification_Dialog();
				notif.handleNotification(aNewItem,"updated",aOldItem);
				this.mEventSynchronizer.notifyItemParticipationChanged(this.mOBMCalendar, aNewItem);
			} else if (diffLevel == 2) {
				this.logger.debug("onModifyItem: this event has had at least one major change", false, false);
				var notif = new Calendar_Email_Notification_Dialog();
				notif.handleNotification(aNewItem,"updated",aOldItem);
				this.mEventSynchronizer.notifyItemModified(this.mOBMCalendar, aNewItem);
			} else {
				this.logger.debug("onModifyItem: no major change and no participation change", false, false);
				// ignore the modification ?
			}
		} else if (aNewItem instanceof Ci.calITodo) {
			this.logger.debug("onModifyItem: this item is a 'todo'", false, false);
			this.mTaskSynchronizer.notifyItemModified(this.mOBMCalendar, aNewItem);
		}
	},

	onDeleteItem: function(aDeletedItem) {
		this.logger.debug("onDeleteItem", false, false);

		if (aDeletedItem instanceof Ci.calIEvent) {
			var notif = new Calendar_Email_Notification_Dialog();
			notif.handleNotification(aDeletedItem,"removed");
			this.mEventSynchronizer.notifyItemDeleted(this.mOBMCalendar, aDeletedItem);
// 			UtilsOBM.storeEventCancelled(aDeletedItem);
		} else if (aDeletedItem instanceof Ci.calITodo) {
			this.mTaskSynchronizer.notifyItemDeleted(this.mOBMCalendar, aDeletedItem);
		}
	},

	onAlarm: function(aAlarmItem) {
		this.logger.debug("onAlarm", false, false);
	},

	onError: function(aCalendar, aErrNo, aMessage) {
		this.logger.debug("onError", false, false);
		this.logger.debug("onError: aErrNo: " + aErrNo + " aMessage: " + aMessage, false, false);
	},

	onPropertyChanged: function(aCalendar, aName, aValue, aOldValue) {
		//this.mEventSynchronizer.notifyItemParticipationChanged(this.mOBMCalendar, aNewItem);
		this.logger.debug("onPropertyChanged (" + "aName: " + aName + ", aValue: " + aValue + ", aOldValue: " + aOldValue + ")", false, false);
	},

	onPropertyDeleting: function(aCalendar, aName) {
		this.logger.debug("onPropertyDeleting (aName: " + aName + ")", false, false);
	}

};

/****
 **** module registration
 ****/

var calOBMCalendarModule = {

	mCID: Components.ID("{86cb9e4f-44c6-4734-9aed-a22795e92b1e}"),
	mContractID: "@mozilla.org/calendar/calendar;1?type=obm",

	registerSelf: function(compMgr, fileSpec, location, type) {
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(this.mCID, "Calendar OBM provider", this.mContractID, fileSpec, location, type);
	},

	getClassObject: function(compMgr, cid, iid) {
		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}

		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}

		return this.mFactory;
	},

	mFactory: {
		createInstance: function(outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new calOBMCalendar()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return calOBMCalendarModule;
}
