/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - 2009-2011 Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */

const Cc = Components.classes;
const Ci = Components.interfaces;


function abEnumerator(list, manager) {
	this._list = list;
	this._index = -1;
	this._manager = manager;
	this._current = null;
}
abEnumerator.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) && !aIID.equals(Ci.nsISimpleEnumerator)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	hasMoreElements : function () {
		if(this._index+1 < this._list.length) {
			try {
				var abid = this._list[this._index+1];
				this._current = this._manager.getAddressBook(abid);
				return true;
			}
			catch(e) {
			}
		}
		return false;
	},
	getNext : function() {
		if (this._current == null)
			return null;
		this._index ++;
		var c = this._current;
		this._current = null;
		return c;
	}
}



/**
 * component to access to a TB3 database
 */
function ObmAddressBookTB3 (aUid) {
	this.mUid = aUid;
	this.abURI = "moz-abmdbdirectory://alia-obm-" + aUid + ".mab";
	this.mDirectory = null; // nsIAbDirectory

	var addressBookManager = Components.classes["@mozilla.org/abmanager;1"]
									.getService(Components.interfaces.nsIAbManager);
	var dirPrefId = "ldap_2.servers." + aUid; //UtilsOBM.getCharPref("extensions.obm.addressbooks." + aUid);
	var iter = addressBookManager.directories;
	while (iter.hasMoreElements()) {
		var data = iter.getNext();
		if (data instanceof Ci.nsIAbDirectory) {
			if (data.dirPrefId == dirPrefId) {
				this.mDirectory = data;
				break;
			}
		}
	}
	if (this.mDirectory === null)
		throw "The addressbook "+aUid+" doesn't exists";
}


ObmAddressBookTB3.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) && !aIID.equals(Ci.obmIAddressBook)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	get uid() {
		return this.mUid;
	},

	get directory () {
		return this.mDirectory;
	},

	getCard : function (aId) {
		var aContact = this.mDirectory.getCardFromProperty("ObmId", aId, false);
		if (!aContact) {
			throw ("Unexpected id (in addressbook " + this.mUid + "): " + aId);
		}
		return aContact;
	},

	modifyCard : function (aCard) {
		this.mDirectory.modifyCard(aCard);
	},

	addCard : function (aCard) {
		this.mDirectory.addCard(aCard);
	},

	deleteCard : function (aCard) {
		var cardsToDelete = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
		cardsToDelete.appendElement(aCard, false);
		this.directory.deleteCards(cardsToDelete);
	},
	cardForEmailAddress : function (aEmail) {
		return this.mDirectory.cardForEmailAddress(aEmail);
	},
	containsCard : function (aCard) {
		try {
			if (this.mDirectory.cardForEmailAddress(aCard.primaryEmail)) {
				return true;
			}
		}
		catch(e) { // we may have an NS_ERROR_NOT_AVAILABLE on asynchronous ab
		}
		return false;
 	}
}

/**
 * component to access to a TB2 database
 */
function ObmAddressBookTB2 (aUid) {
	this.mUid = aUid;
	this.mDirectory = null;
	this.mMDBDirectory = null;
	this.abURI = "moz-abmdbdirectory://alia-obm-" + aUid + ".mab";
	this.abService = Cc["@mozilla.org/addressbook;1"].getService(Ci.nsIAddressBook);
	this.addrDb = this.abService.getAbDatabaseFromURI(this.abURI);

	this.RDFService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);

	try {
		this.mDirectory = this.RDFService.GetResource(this.abURI).QueryInterface(Ci.nsIAbDirectory);
		this.mMDBDirectory = this.RDFService.GetResource(this.abURI).QueryInterface(Ci.nsIAbMDBDirectory);
		
		var children = this.mDirectory.childCards; // it triggers an exception if no directory or no addressbook ?

	}catch(e) {
		throw "The addressbook "+aUid+" doesn't exists";
	}
}


ObmAddressBookTB2.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) && !aIID.equals(Ci.obmIAddressBook)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	get uid() {
		return this.mUid;
	},

	get directory () {
		return this.mDirectory;
	},

	getCard : function (aId) {
		var contact = this.addrDb.getCardFromAttribute(this.directory, "AnniversaryDay", aId, false);
		if (!contact) {
			throw ("Unexpected id (in addressbook " + this.mUid + "): " + aId);
		}
		return contact;
	},

	modifyCard : function (aCard) {
		this.addrDb.editCard(aCard, true, null);
		this.addrDb.commit(Ci.nsAddrDBCommitType.kSessionCommit);
	},

	addCard : function (aCard) {
		this.addrDb.createNewCardAndAddToDB(aCard, true);
		this.addrDb.commit(Ci.nsAddrDBCommitType.kSessionCommit);
	},

	deleteCard : function (aCard) {
		this.addrDb.deleteCard(aCard, true, null);
		this.addrDb.commit(Ci.nsAddrDBCommitType.kSessionCommit);
	},

	cardForEmailAddress : function (aEmail) {
		return this.mMDBDirectory.cardForEmailAddress(aEmail);
	},
	containsCard : function (aCard) {
		return this.mMDBDirectory.cardForEmailAddress(aCard.primaryEmail);
 	}
}

/**
 * main object to access to OBM address books
 */
function ObmAddressBooksManager () {
	// Init logger
	this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
	var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
	this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
	this.logger.setHeader("ObmAddressBooksManager.js: ");
	this.logger.debug("new ObmAddressBooksManager()", false, false);

	this.abVersion = Cc["@mozilla.org/abmanager;1"] ? 3 : 2;
	if (this.abVersion == 3) {
		this.addressBookManager = Components.classes["@mozilla.org/abmanager;1"]
									.getService(Components.interfaces.nsIAbManager);
	}
};

ObmAddressBooksManager.prototype = {

	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) && !aIID.equals(Ci.obmIAddressBooksManager)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	_addAddressBookNames: function(newName) {
		var namesPref = UtilsOBM.getCharPref("extensions.obm.addressbooks");
		var names = namesPref ? namesPref.split(",") : [];
		var value = "";
		if (names.length == 0) {
			value = newName;
		} else {
			for(var i=0; i<names.length; i++) {
				if (names[i] == newName) {
					return;
				}
			}
			value = namesPref + "," + newName;
		}
		UtilsOBM.setCharPref("extensions.obm.addressbooks", value);
	},

	_removeAddressBookNames: function(name) {
		var namesPref = UtilsOBM.getCharPref("extensions.obm.addressbooks");
		var names = namesPref ? namesPref.split(",") : [];
		var newNames = [];
		for (var i=0; i<names.length; i++) {
			if (names[i] != name) {
				newNames.push(names[i]);
			}
		}
		UtilsOBM.setCharPref("extensions.obm.addressbooks", newNames.join(","));
	},

	getAddressBookUids: function(count) {
		var names = UtilsOBM.getCharPref("extensions.obm.addressbooks");
		var list = names ? names.split(",") : [];
		if (count)
			count.value = list.length;
		return list;
	},

	getObmAddressBooks : function () {
		var names = UtilsOBM.getCharPref("extensions.obm.addressbooks");
		var list = names ? names.split(",") : [];
		var enumerator = new abEnumerator(list, this);
		return enumerator;
	},

	getAddressBook : function (aUid) {
		if (this.abVersion == 2) {
			return (new ObmAddressBookTB2(aUid)).QueryInterface(Ci.obmIAddressBook);
		}
		else {
			return (new ObmAddressBookTB3(aUid)).QueryInterface(Ci.obmIAddressBook);
		}
	},

	getOBMContactsAddressBook : function () {
		this.logger.debug("getOBMContactsAddressBook: ",false,false);
		var list = this.getObmAddressBooks();

		var expectedName = UtilsOBM.getCharPref("extensions.obm.addressbooks.contacts", "OBM - Contacts");

		// the only way to find the obm contact address book, is to check
		// its name. However, the user could change the name, so
		// we check also if the name does not contain the word "contact"
		var found = null;
		var possible = null;
		var first = null;

		while (list.hasMoreElements()) {
			var ad = list.getNext().QueryInterface(Ci.obmIAddressBook);
			if (!first)
				first = ad;
			var name = ad.directory.dirName;
			if (name == expectedName) { // original name
				found = ad;
				break;
			}
			
			if (name.search(/contact/i) != -1 && name.search(/collect/i) == -1) {
				// here we want to add the contact to the addressbook of the current user.
				// unfortunately there is a way to share addressbooks.
				// so a user having another user addressbook will have somethin like :
				// Mes Contacts
				// Mes contacts (other user)
				// So in the "possible" list we keep the one having the less characters
				// CF Jira OBMFULL-2328
				if ( possible == null || possible.directory.dirName.length > name.length ) {
					possible = ad;
				}
			}
		}

		if (!found) {
			if (possible)
				found = possible;
			else if (first) // ok, we don't have the choice, let's take the first OBM ad
				found = first;
		}
		return found;
	},

	removeAddressBook : function (aUid) {
		if (this.abVersion == 2) {
			var addr = new ObmAddressBookTB2(aUid);
			var datasource = Components.classes["@mozilla.org/rdf/datasource;1?name=addressdirectory"]
				.getService(Components.interfaces.nsIRDFDataSource);

			var parentArray = Components.classes["@mozilla.org/supports-array;1"]
							.createInstance(Components.interfaces.nsISupportsArray);
			parentArray.AppendElement(addr.RDFService.GetResource("moz-abdirectory://"));

			var directoryResource = addr.RDFService.GetResource(addr.abURI);

			var resourceArray = Components.classes["@mozilla.org/supports-array;1"]
				.createInstance(Components.interfaces.nsISupportsArray);
			resourceArray.AppendElement(addr.mDirectory);
			addr.abService.deleteAddressBooks(datasource, parentArray, resourceArray);
		}
		else {
			var addr = new ObmAddressBookTB3(aUid);
			if (addr.directory) {
				this.addressBookManager.deleteAddressBook(addr.directory.URI);
			}
		}
		this._removeAddressBookNames(aUid);
	},

	ensureAddressBookExists : function (aUid, aName, aOwnerDisplayName) {
		this.logger.debug("ensureAddressBookExists()", false, false);

		/* Get configured name for "contacts" and "collected_contacts" */
		var dirName = UtilsOBM.getCharPref("extensions.obm.addressbooks." + aName, aName);

		if (aOwnerDisplayName != null && aOwnerDisplayName != "") {
			dirName += " (" + aOwnerDisplayName + ")";
		}

		if (this.abVersion == 3) {

			try {
				var addrb = new ObmAddressBookTB3(aUid);
				if (addrb.directory.dirName != dirName)
					addrb.directory.dirName = dirName;
			} catch(e) {
				var abURI = "moz-abmdbdirectory://" + aUid + ".mab";
				var dirPrefId = "ldap_2.servers." + aUid;
				this.addressBookManager.newAddressBook(dirName, abURI, 2, dirPrefId);
				this._addAddressBookNames(aUid);
			}

		} else {
			try {
				var addrb = new ObmAddressBookTB2(aUid);

				// Ok, address books are here, skiping creation
				/* Update name if necessary TODO: test it with TB2 */
				if (addrb.directory.dirName != dirName)
					addrb.directory.dirName = dirName;
			}
			catch(e) {
				if (e.name != "NS_ERROR_FILE_NOT_FOUND")
					throw e;

				var abURI = "moz-abmdbdirectory://alia-obm-" + aUid + ".mab";
				var directoryProperties = Cc["@mozilla.org/addressbook/properties;1"]
											.getService(Ci.nsIAbDirectoryProperties);
				directoryProperties.description = dirName;
				// What 2 is ?
				directoryProperties.dirType = 2;
				directoryProperties.fileName = abURI.split('/').pop();
				var addressBookService = Cc["@mozilla.org/addressbook;1"].getService(Ci.nsIAddressBook);
				addressBookService.newAddressBook(directoryProperties);
				this._addAddressBookNames(aUid);
			}
		}
	},
	escapedVCardToAbCard : function (aStringVcard) {
		if (this.abVersion == 3) {
			return this.addressBookManager.escapedVCardToAbCard(aStringVcard);
		}
		else {
			var ad = Components.classes["@mozilla.org/addressbook;1"]
					.createInstance(Components.interfaces.nsIAddressBook);
			var vcard = ad.escapedVCardToAbCard(aStringVcard);
			
			// in TB2, escapedVCardToAbCard return a simple implementation of vcard
			// and we need a mdb vcard implementation for some operations,
			// like containsCard
			var mdbVCard = Components.classes["@mozilla.org/addressbook/moz-abmdbcard;1"]
							.createInstance(Components.interfaces.nsIAbCard);
			mdbVCard.copy(vcard);
			return mdbVCard;
		}
 	}
};


var ObmAddressBooksModule = {
	mCID: Components.ID("{f6cdfa9e-6961-4d9e-a7cc-612783c1bb0b}"),
	mContractID: "@obm.org/addressbooksmanager;1",

	registerSelf: function(compMgr, fileSpec, location, type) {
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(this.mCID, "OBM AddressBooks Manager", this.mContractID, fileSpec, location, type);
	},
	getClassObject: function(compMgr, cid, iid) {
		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}
		// Load UtilsOBM.js if needed
		if (!this._utilsLoaded) {
			var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader);
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/UtilsOBM.js");
			this._utilsLoaded = true;
		}
		return this.mFactory;
	},
	mFactory: {
		createInstance: function(outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new ObmAddressBooksManager()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return ObmAddressBooksModule;
}


