/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  http://www.obm.org/
 *
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

function _appendXMLElement(aRootElement, aChildElementName, aChildElementValue) {
	if ( aChildElementValue === undefined || aChildElementValue === null ) {
	  //append nothing
	} else {
	  aRootElement[aChildElementName] = aChildElementValue;
	}
}

function _xS(value) {
	// FIXME : toString() ?
	return (value.length() == 1) ? value + "" : "";
}

function _xL(value, def) {
	if ( typeof value == "number" ) {
		return parseInt(value, 10);
	}

	if (value.length() == 1)
		return parseInt(value + "", 10);
	if (def)
		return def;
	return 0;
}

function _xD(value) {
	return (value.length() == 1) ? parseFloat(value + "") : 0;
}

function _xB(value) {
	return value + "" == "true";
}

function _eventShell(aEvent, aKind) {
  aEvent.QueryInterface(Ci.obmIREventRecurrence);
  if (aKind == "event") {
		var obj = <event/>; // xmlns="http://www.obm.org/xsd/sync/event.xsd"/>;
		if (aEvent.uid) {
			obj.@id = aEvent.uid;
		}
		if (aEvent.isInternal)
			obj.@isInternal = "true";
		else
			obj.@isInternal = "false";

		if ( "generation" in aEvent && aEvent.generation !== null ) {
			obj.@sequence = _xL(aEvent.generation);
		} else {
			obj.@sequence = 0;
		}

		_appendXMLElement(obj, "extId", aEvent.extId);
  } else {
		var obj = <eventException/>
		_appendXMLElement(obj, "recurrenceId", aEvent.recurrenceId);
  }

  obj.@allDay = aEvent.allday;
  obj.@type = "VEVENT";

  _appendXMLElement(obj, "owner", aEvent.owner);
  _appendXMLElement(obj, "priority" , aEvent.priority);
  _appendXMLElement(obj, "privacy" , aEvent.privacy);
  _appendXMLElement(obj, "date" , aEvent.date);
  _appendXMLElement(obj, "duration" , aEvent.duration);
  _appendXMLElement(obj, "title" , aEvent.title);
  _appendXMLElement(obj, "description" , aEvent.description);
  _appendXMLElement(obj, "category" , aEvent.category);
  _appendXMLElement(obj, "location" , aEvent.location);
  _appendXMLElement(obj, "alert" , aEvent.alert);
  _appendXMLElement(obj, "opacity" , aEvent.opacity);
  _appendXMLElement(obj, "ownerEmail", aEvent.ownerEmail);

  var atts = aEvent.getAttendees({});
  for (var i = 0; i < atts.length; i++) {
	var att = <attendee email={atts[i].email} displayName={atts[i].displayName} state={atts[i].status} required={atts[i].type}/>;
	if (atts[i].isOrganizer)
		att.@isOrganizer ='true';
    obj.attendees.attendee[i] = att;
  }

  obj.recurrence = <recurrence kind={aEvent.kind} freq={aEvent.frequence} days={aEvent.days}/>;
  if (aEvent.end)
  	obj.*::recurrence.@end = aEvent.end;

  var exs =	aEvent.getExceptions({});
  for (var i = 0; i < exs.length; i++) {
  	obj.recurrence.exceptions.exception[i] = exs[i];
  }

  var eventExs = aEvent.getEventExceptions({});
  for (var i = 0; i < eventExs.length; i++) {
  	obj.recurrence.eventExceptions.eventException[i] = _eventShell(eventExs[i], "eventException");
  }

  return obj;
}

function JSObjEventShell(aEvent /* obmIREvent */) {
  var obj = _eventShell(aEvent, "event");
  return ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + obj.toString());
}

function ObmRCalendarProxy() {
  // Init logger
  this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
  var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
  this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
  this.logger.setHeader("obmCalendarProxy: ");
  this.logger.debug("new obmCalendarProxy()", false, false);

	this.RESTClient = Cc["@obm.org/restclient;1"].getService().wrappedJSObject;

	this._listeners = [];
	this._url = null;
	this._loginUrl = null;
	this._token = {
		session_id: null
	};
};

ObmRCalendarProxy.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIRCalendarProxy) &&
			!aIID.equals(Ci.obmIRProxyBase)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	/*
	 * obmIRProxyBase
	 */
	addListener: function(listener) {
		for (var i = 0; i < this._listeners.length; ++i) {
			if (this._listeners[i] == listener)
				return;
		}
		this._listeners.push(listener);
	},

	removeListener: function(listener) {
		this._listeners = [];
	},
	_notifyListeners: function(aResultCode, aMethodName, aObj, aData) {
		// A copy is needed, a new listeners could has been added during the
		// callbacks
		var listeners = this._listeners.slice(0, this._listeners.length);
		for (var i = 0; i < listeners.length; ++i) {
			try {
				listeners[i].onResponse(aResultCode, aMethodName, aObj, aData);
			} catch(e) {
				// Continue to notify listeners
				this.logger.error(e + " (ignored)");
			}
		}
	},

	get token() {
		return this._token;
	},
	set token(aToken) {
		this._token = aToken;
	},

	get PortUrl() {
		return this._url;
	},
	set PortUrl(aURL) {
		if(aURL.charAt(aURL.length -1) == '/') {
		  aURL = aURL.substring(0, aURL.length - 1);
		}
		this._url = aURL + "/calendar";
		this._loginUrl = aURL + "/login";
	},

	/*
	 * obmIRCalendarProxy
	 */
	_callback: function(aMethodName, aObj, aXml, aData) {
		this.logger.debug("_callback(aMethodName = " + aMethodName + ")", false, false);
		if (aObj && aObj.name == "Error") {
			var msg = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);

			if (aMethodName == "doLogin" &&  aObj.fileName != "Connection failure") {
				msg.data = "No token available";
			} else {
				msg.data = aObj.fileName;
			}
		  this._notifyListeners(Cr.NS_ERROR_FAILURE, aMethodName, msg, aData);
		  return;
		}
		switch(aMethodName) {
			case "doLogin":
				var sid = _xS(aXml.*::sid);
				this._token.session_id = sid;
				this.logger.debug("token obtained with sid:"+sid, false, false);
				var syncVersion = null;
				if (aXml.*::version != undefined) {
				  syncVersion = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
				  syncVersion.data = aXml.*::version.@major
									  + "." + aXml.*::version.@minor
									  + "." + aXml.*::version.@release;
				  this.logger.debug("sync version:" + syncVersion.data, false, false);
				}
				var domain = _xS(aXml.*::domain);
				UtilsOBM.setCharPref("extensions.obm.server.domain", domain);
				this._notifyListeners(Cr.NS_OK, aMethodName, syncVersion, aData);
				break;
			case "getSync":
				this._notifyListeners(Cr.NS_OK, aMethodName, new GetSyncEventResponseShell(aXml), aData);
				break;
			case "getSyncWithSortedChanges":
				this._notifyListeners(Cr.NS_OK, aMethodName, new GetSyncEventWithSortedChangesResponseShell(aXml), aData);
				break;
			case "createEvent":
				var id = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
				id.data = _xS(aXml.*::value);
				this._notifyListeners(Cr.NS_OK, aMethodName, id, aData);
				break;
			case "modifyEvent":
				this._notifyListeners(Cr.NS_OK, aMethodName, new EventShell(aXml), aData);
				break;
			case "changeParticipationState":
				this._notifyListeners(Cr.NS_OK, aMethodName, null, aData);
				break;
			case "removeEvent":
				this._notifyListeners(Cr.NS_OK, aMethodName, null, aData);
				break;
			case "removeEventByExtId":
				this._notifyListeners(Cr.NS_OK, aMethodName, null, aData);
				break;
			case "listCalendars":
				var calendarInfos = this._makeCalendarInfoArray(aXml);
				this._notifyListeners(Cr.NS_OK, aMethodName, calendarInfos, aData);
				break;
			case "listCategories":
				this._notifyListeners(Cr.NS_OK, aMethodName, new GetCategoriesShell(aXml), aData);
				break;
			case "purge":
				this._notifyListeners(Cr.NS_OK, aMethodName, null, aData);
				break;
			case "getCalendarMetadata":
				var calendarInfos = this._makeCalendarInfoArray(aXml);
				this._notifyListeners(Cr.NS_OK, aMethodName, calendarInfos, aData);
				break;
			default:
				this._notifyListeners(Cr.NS_ERROR_FAILURE, aMethodName, null, aData);
		};
	},

	getSync: function(aCalName, aLastSync, aData) {
	this.logger.debug("getSync()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		p.add("calendar", aCalName);
		p.add("lastSync", aLastSync);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "getSync", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	getSyncWithSortedChanges: function(aCalName, aLastSync, aData) {
	this.logger.debug("getSyncWithSortedChanges()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		p.add("calendar", aCalName);
		p.add("lastSync", aLastSync);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "getSyncWithSortedChanges", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	createEvent: function(aCalendarId, aEvent, aNotification, aData) {
		this.logger.debug("createEvent()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		p.add("calendar", aCalendarId);
		p.add("event", JSObjEventShell(aEvent));
		p.add("notification", aNotification);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "createEvent", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	modifyEvent: function(aCalendarId, aEvent, aUpdateAttendees, aNotification, aGeneration, aData) {
		this.logger.debug("modifyEvent()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		p.add("calendar", aCalendarId);
		p.add("event", JSObjEventShell(aEvent));
		p.add("updateAttendees", aUpdateAttendees);
		p.add("notification", aNotification);
		p.add("sequence", aGeneration);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "modifyEvent", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	changeParticipationState: function(calendar, extId, participationState, aNotification, aGeneration, aData) {
		this.logger.debug("changeParticipationState()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		p.add("calendar", calendar);
		p.add("extId", extId);
		p.add("state", participationState);
		p.add("notification", aNotification);
		p.add("sequence", aGeneration);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "changeParticipationState", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	removeEvent: function(aCalendarId, aUid, aNotification, aGeneration, aData) {
		this.logger.debug("removeEvent()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		p.add("calendar", aCalendarId);
		p.add("id", aUid);
		p.add("notification", aNotification);
		p.add("sequence", aGeneration);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "removeEvent", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	removeEventByExtId: function(aCalendarId, aExtId, aNotification, aGeneration, aData) {
		this.logger.debug("removeEventByExtId()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		p.add("calendar", aCalendarId);
		p.add("extId", aExtId);
		p.add("notification", aNotification);
		p.add("sequence", aGeneration);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "removeEventByExtId", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	doLogin: function(aLogin, aPwd, aOrigin, aData) {
		this.logger.debug("doLogin()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("login", aLogin);
		p.add("password", aPwd);
		p.add("origin", aOrigin);
		// closure
		var self = this;
		this.RESTClient.invoke(this._loginUrl, "doLogin", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	listCalendars: function(aData) {
		this.logger.debug("listCalendars()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "listCalendars", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	listCategories: function(aData) {
		this.logger.debug("listCategories()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "listCategories", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml);}, aData);
	},
	getCalendarMetadata: function(calendarEmails, calendarEmailsLength, aData) {
		this.logger.debug("getCalendarMetadata()", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);

		for (var i = 0 ; i < calendarEmails.length ; i++)
			p.add("calendar", calendarEmails[i]);

		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "getCalendarMetadata", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml);}, aData);
	},
	// void purge(in AString calendar, in nsISupports aData);
	purge: function(aCalName, aData) {
		this.logger.debug("purge(" + aCalName + ")", false, false);
		var p = this.RESTClient.newRESTClientParameters();
		p.add("sid", this._token.session_id);
		p.add("calendar", aCalName);
		// closure
		var self = this;
		this.RESTClient.invoke(this._url, "purge", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	_makeCalendarInfoArray: function(aXml) {
		// Makes an XPCOM array
		var infoArray = {
			list: aXml.*::info,
			QueryInterface: function(aIID) {
				if (!aIID.equals(Ci.nsISupports) &&
					!aIID.equals(Ci.nsIArray)) {
					throw Cr.NS_ERROR_NO_INTERFACE;
				}
				return this;
			},
			get length() {return this.list.length();},
			enumerate: function() {throw Cr.NS_ERROR_NOT_IMPLEMENTED},
			indexOf: function(idx, elt) {throw Cr.NS_ERROR_NOT_IMPLEMENTED},
			queryElementAt: function(idx, uuid) {
				var cal = new CalendarShell(this.list[idx]);
				return cal.QueryInterface(uuid);
			}
		};
		return infoArray;
	}
};

function CalendarShell(aXml) {
	this._mXml = aXml;
}

CalendarShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIRCalendarInfo)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	// obmIRCalendarInfo
	/*<info>
		<first>user3</first>
		<last>user3</last>
		<mail>user3@dom1.local</mail>
		<uid>user3</uid>
		<read>true</read>
		<write>true</write>
	</info>*/
	get uid() {return _xS(this._mXml.*::uid)},
	get firstname() {return _xS(this._mXml.*::first)},
	get lastname() {return _xS(this._mXml.*::last)},
	get mail() {return _xS(this._mXml.*::mail)},
	get read() {return _xB(this._mXml.*::read)},
	get write() {return _xB(this._mXml.*::write)}
};


function ObmRAttendee(email, displayName, status, type, isOrganizer) {
	this._email = email;
	this._displayName = displayName;
	this._status = status;
	this._type = type;
	this._isOrganizer = isOrganizer;
}
ObmRAttendee.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIRAttendee)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	get email () { return this._email;},
	get displayName () { return this._displayName;},
	get status () { return this._status;},
	set status (val) { this._status = val;},
	get type () { return this._type;},
	get isOrganizer () { return this._isOrganizer;}
}



function ParticipationChangeShell(aXml) {
	this._mXml = aXml;
	this._attendees = null; // cache for attendees
}

ParticipationChangeShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIRParticipationChange)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	_readAttendees : function() {
		this._attendees = [];
		if (this._mXml.*::attendees) {
			for each( var att in this._mXml.*::attendees.*) {
				var attendee = new ObmRAttendee(_xS(att.@email),
												_xS(att.@displayName),
												_xS(att.@state),
												_xS(att.@required),
												_xB(att.@isOrganizer));
				this._attendees.push(attendee);
			}
		}
	},
	// obmIRParticipationChange
	get uid() {return this._mXml.@id},
	get generation() {return this._mXml.@sequence ? _xL(this._mXml.@sequence) : 0 },
	get extId() {return this._mXml.@extId},
	getAttendees: function(count) {
		if (this._attendees === null) {
			this._readAttendees();
		}
		count.value = this._attendees.length;
		return this._attendees;
	},

	getAttendeeByEmail : function(email) {
		if (this._attendees === null) {
			this._readAttendees();
		}
		for(var i=0; i < this._attendees.length; i++) {
			if (this._attendees[i].email.toLowerCase() == email.toLowerCase())
				return this._attendees[i];
		}
		return null;
	}
};

function EventShell(aXml) {
	this._mXml = aXml;
	this._attendees = null; // cache for attendees
}

EventShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIREventRecurrence) &&
			!aIID.equals(Ci.obmIREvent)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	_readAttendees : function() {
		this._attendees = [];
		if (this._mXml.*::attendees) {
			for each( var att in this._mXml.*::attendees.*) {
				var attendee = new ObmRAttendee(_xS(att.@email),
												_xS(att.@displayName),
												_xS(att.@state),
												_xS(att.@required),
												_xB(att.@isOrganizer));
				this._attendees.push(attendee);
			}
		}
	},
	// obmIREvent
	get uid() {return this._mXml.@id},
	get extId() {return this._mXml.*::extId},
	get generation() {return this._mXml.@sequence ?  _xL(this._mXml.@sequence)  : 0; },
	get recurrenceId() {return this._mXml.*::recurrenceId},
	get owner() {return _xS(this._mXml.*::owner)},
	get ownerEmail() {return _xS(this._mXml.*::ownerEmail)},
	get priority() {return _xL(this._mXml.*::priority)},
	get privacy() {return _xL(this._mXml.*::privacy)},
	get allday() {return _xB(this._mXml.@allDay)},
	get date() {return _xS(this._mXml.*::date)},
	get duration() {return _xL(this._mXml.*::duration)},
	get title() {return _xS(this._mXml.*::title)},
	get description() {return _xS(this._mXml.*::description)},
	get category() {return _xS(this._mXml.*::category)},
	get location() {return _xS(this._mXml.*::location)},
	get opacity() {return _xS(this._mXml.*::opacity)},
	get recurrence() {return this;},
	getAttendees: function(count) {
		if (this._attendees === null) {
			this._readAttendees();
		}
		count.value = this._attendees.length;
		return this._attendees;
	},

	getAttendeeByEmail : function(email) {
		if (this._attendees === null) {
			this._readAttendees();
		}
		for(var i=0; i < this._attendees.length; i++) {
			if (this._attendees[i].email.toLowerCase() == email.toLowerCase())
				return this._attendees[i];
		}
		return null;
	},

	// obmIREventRecurrence
	get kind() {return _xS(this._mXml.*::recurrence.@kind)},
	get frequence() {return _xL(this._mXml.*::recurrence.@freq)},
	get days() {return _xS(this._mXml.*::recurrence.@days)},
	get end() {return _xS(this._mXml.*::recurrence.@end)},
	getExceptions: function(count) {
		var exceptions = [];
		var exs = this._mXml.*::recurrence.*::exceptions.*::exception.*;
		for each (var ex in exs) {
			exceptions.push(_xS(ex));
		}
		if (exceptions.length > 0) {
			count.value = exceptions.length;
			return exceptions;
		} else {
			return;
		}
	},
	getEventExceptions: function(count) {
	  var eventExceptions  = [];
	  var eventExs = this._mXml.*::recurrence.*::eventExceptions.*;
	  for each (var eventEx in eventExs) {
		  eventExceptions.push(new EventShell(eventEx));
	  }
	  if (eventExceptions.length > 0) {
		  count.value = eventExceptions.length;
		  return eventExceptions;
	  } else {
		  return;
	  }
	},
	get alert() {return _xL(this._mXml.*::alert, -1)},
	get isInternal() {return (this._mXml.@isInternal == 'true');}
};

function GetSyncEventWithSortedChangesResponseShell(aXml) {
	this._updated = [];
	this._removed = [];
	this._participationChanged = [];
	this._removedExtId = [];
	this._lastSync = aXml.@lastSync.toString();

	for each (var updated in aXml.*::updated.*) {
	    if ( (updated.*::recurrenceId == undefined) || (updated.*::recurrenceId == 0)) {
		  this._updated.push(new EventShell(updated));
		} else {
		  //skip this is an eventException allready in parent item
		}
	}

	for each (var removed in aXml.*::removed.*) {
		this._removed.push(removed.@id.toString());
		if (removed.@extId != undefined) {
			this._removedExtId.push(removed.@extId.toString());
		}
	}

	for each (var participationChange in aXml.*::participationChanges.*) {
		this._participationChanged.push(new ParticipationChangeShell(participationChange));
	}


	/*
	<calendar-changes lastSync="20081127230000000" xmlns="http://www.obm.org/xsd/sync/calendar-changes.xsd">
		<participation-changes>
			<participation extId="966c71d7-04d0-4551-85b7-c9641e20f1d6" id="26">
			<attendees>
				<attendee email="blum@obm.matthieu.lng" state="ACCEPTED"/>
				<attendee email="jaures@obm.matthieu.lng" state="ACCEPTED"/>
			</attendees>
			</participation>
		</participation-changes>
		<removed>
			<event id="user3-2" extId="85f148ae-2400-4860-b1dd-ae3fa062b07b"/>
			<event id="user3-3" extId="e1e253f9-b7a6-488b-a360-5ca294afb803"/>
			<event id="user3-4" extId="ad3f09e7-1a5e-40c2-ae1d-12906fbc3760"/>
			<event id="user3-5" extId="e19991ea-8131-4b61-88fe-e25b84311073"/>
		</removed>
		<updated>
			<event id="1">
			<title>eeeeee</title>
			<owner>user3</owner>
			<date>20081127110000000</date>
			<duration>3600</duration>
			<priority>2</priority>
			<privacy>0</privacy>
			<attendees>
				<attendee displayName="user3 user3" email="user3@dom1.local" required="CHAIR" state="ACCEPTED"/>
			</attendees>
			<recurrence kind="none"/>
			</event>
		</updated>
	</calendar-changes>
	*/
}

GetSyncEventWithSortedChangesResponseShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIRCalendarSyncWithSortedChangesResponse)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	getUpdatedEvents: function(count) {
		count.value = this._updated.length;
		return this._updated;
	},
	getRemovedEventIds: function(count) {
		count.value = this._removed.length;
		return this._removed;
	},
	getRemovedEventExtIds: function(count) {
		count.value = this._removedExtId.length;
		return this._removedExtId;
	},
	getParticipationChanges: function(count) {
		count.value = this._participationChanged.length;
		return this._participationChanged;
	},
	get lastsync() {
		return this._lastSync;
	}
};

function GetSyncEventResponseShell(aXml) {
	this._updated = [];
	this._removed = [];
	this._removedExtId = [];
	this._lastSync = aXml.@lastSync.toString();

	for each (var updated in aXml.*::updated.*) {
// 	    if ( (updated.*::recurrenceId == undefined) || (updated.*::recurrenceId == 0)) {
		  this._updated.push(new EventShell(updated));
// 		} else {
		  //skip this is an eventException allready in parent item
// 		}
	}

	for each (var removed in aXml.*::removed.*) {
		this._removed.push(removed.@id.toString());
		if (removed.@extId != undefined) {
			this._removedExtId.push(removed.@extId.toString());
		}
	}

	/*
	<calendar-changes lastSync="20081127230000000" xmlns="http://www.obm.org/xsd/sync/calendar-changes.xsd">
		<removed>
			<event id="user3-2" extId="85f148ae-2400-4860-b1dd-ae3fa062b07b"/>
			<event id="user3-3" extId="e1e253f9-b7a6-488b-a360-5ca294afb803"/>
			<event id="user3-4" extId="ad3f09e7-1a5e-40c2-ae1d-12906fbc3760"/>
			<event id="user3-5" extId="e19991ea-8131-4b61-88fe-e25b84311073"/>
		</removed>
		<updated>
			<event id="1">
			<title>eeeeee</title>
			<owner>user3</owner>
			<date>20081127110000000</date>
			<duration>3600</duration>
			<priority>2</priority>
			<privacy>0</privacy>
			<attendees>
				<attendee displayName="user3 user3" email="user3@dom1.local" required="CHAIR" state="ACCEPTED"/>
			</attendees>
			<recurrence kind="none"/>
			</event>
		</updated>
	</calendar-changes>
	*/
}

GetSyncEventResponseShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIRCalendarSyncResponse)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	getUpdatedEvents: function(count) {
		count.value = this._updated.length;
		return this._updated;
	},
	getRemovedEventIds: function(count) {
		count.value = this._removed.length;
		return this._removed;
	},
	getRemovedEventExtIds: function(count) {
		count.value = this._removedExtId.length;
		return this._removedExtId;
	},
	get lastsync() {
		return this._lastSync;
	}
};

function GetCategoriesShell(aXml) {
	this._categories = [];
	for each (var xcat in aXml.*::cat) {
		var cat = {};
		cat.id = _xL(xcat.@id);
		cat.label = _xS(xcat.@label);
		cat.QueryInterface = function(aIID) {
			if (!aIID.equals(Ci.nsISupports) &&
				!aIID.equals(Ci.obmIRCategory)) {
				throw Cr.NS_ERROR_NO_INTERFACE;
			}
			return this;
		};
		this._categories.push(cat);
	}
}

GetCategoriesShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIRCategoriesResponse)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	getCategories: function(count) {
		count.value = this._categories.length;
		return this._categories;
	}
};
// module

var ObmRCalendarProxyModule = {
	mCID: Components.ID("{e2b34be1-b2b2-47de-bfe2-4f8126183ca5}"),
	mContractID: "@obm.org/obm-sync-rest/calendarproxy;1",
	registerSelf: function (compMgr, fileSpec, location, type) {
		dump("registerSelf:@obm.org/obm-sync-rest/calendarproxy;1\n");
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(
			this.mCID,
			"OBM Calendar Proxy",
			this.mContractID,
			fileSpec,
			location,
			type
		);
	},
	getClassObject: function (compMgr, cid, iid) {
		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}
		// Load UtilsOBM.js if needed
		if (!this._utilsLoaded) {
			var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader);
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/UtilsOBM.js");
			this._utilsLoaded = true;
		}
		return this.mFactory;
	},
	mFactory: {
		createInstance: function (outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new ObmRCalendarProxy()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return ObmRCalendarProxyModule;
}
