/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

//Init logger
var gLogger;
var writer;

function RESTClientParameters()
{
	var _pl = new Array();
	this.add = function(name, value)
	{
		if (_pl[name] == null)
			_pl[name] = [value];
		else
			_pl[name].push(value);
		return this;
	}

    /**
     * Contrary to popular belief, doesn't return XML at all. Instead, returns
     * the properties of the object as a string of (URL-escaped) query parameters.
     */
	this.toXml = function()
	{
		var result = "";
		var params = [];
		for (var paramName in _pl) {
		  var paramValues =  _pl[paramName];
		  for (var i = 0 ; i < paramValues.length ; i++) {
			  var elem = paramName + "=";
			  var dparam;
			  if (paramName == "password") {
				dparam = "  " + elem + "******";
			  } else {
				dparam = "  " + elem;
				try {
					dparam += paramValues[i]+"";
				} catch(e) {}
			  }
			  gLogger.debug(dparam, false, false);
			  paramValue = paramValues[i];
			  elem += encodeURIComponent(paramValue);
			  params.push(elem);
		  }
		}
		result = params.join("&");
		return result;	
	}
}

function RESTClient() {}

RESTClient.username = null;
RESTClient.password = null;

RESTClient.invoke = function(url, method, parameters, async, callback, aData)
{
	gLogger.debug("invoke", false, false);
	return RESTClient._sendRESTRequest(url, method, parameters, async, callback, aData);
}

RESTClient._sendRESTRequest = function(url, method, parameters, async, callback, aData)
{
	// build REST request
	gLogger.debug("_sendRESTRequest: " + url + "/" + method + "(", false, false);
	var sr = parameters.toXml();
	gLogger.debug(")", false, false);
	var xmlHttp = RESTClient._getXmlHttp();
	if (RESTClient.userName && RESTClient.password){
		xmlHttp.open("POST", url+"/"+ method, async, RESTClient.userName, RESTClient.password);
		// Some WS implementations (i.e. BEA WebLogic Server 10.0 JAX-WS) don't support Challenge/Response HTTP BASIC, so we send authorization headers in the first request
		xmlHttp.setRequestHeader("Authorization", "Basic " + RESTClient._toBase64(RESTClient.userName + ":" + RESTClient.password));
	}
	else
		xmlHttp.open("POST", url+"/"+ method, async);
	//var RESTaction = ((ns.lastIndexOf("/") != ns.length - 1) ? ns + "/" : ns) + method;
	//xmlHttp.setRequestHeader("RESTAction", RESTaction);
	xmlHttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded; charset=utf-8");
	//xmlHttp.setRequestHeader("Content-Type", "text/xml; charset=utf-8");
	if(async) 
	{
		xmlHttp.onerror = function() {
			callback(method, new Error(500, "Connection failure"), null, aData);
		}
		xmlHttp.onreadystatechange = function() { if(xmlHttp.readyState == 4) { if (xmlHttp.responseXML)
					RESTClient._onSendRESTRequest(method, async, callback, xmlHttp, aData);
					else {
						callback(method, new Error(500, "No such method"), null, aData);
					}
					
			}
		}
	}
	xmlHttp.send(sr);
	if (!async)
		return RESTClient._onSendRESTRequest(method, async, callback, xmlHttp, aData);
}

RESTClient._onSendRESTRequest = function(method, async, callback, req, aData) 
{
	var o = null;
	
	gLogger.debug("_onSendRESTRequest: "+method + " => " + req.responseText, false, false);
	if(req.responseXML.getElementsByTagName("error").length > 0)
	{
		var errorStr;
		errorStr = req.responseXML.getElementsByTagName("message")[0].childNodes[0].nodeValue;
		o = new Error(500, errorStr);
	}	else {
		o = "ok";
	}
	if(callback) {
		callback(method, o, new XML(req.responseText.replace(/<\?xml.*\?>/, "")), aData);
	}
	if(!async)
		return o;
}

// private: xmlhttp factory
RESTClient._getXmlHttp = function() 
{
	var req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
	req.QueryInterface(Ci.nsIJSXMLHttpRequest);
	if(req.readyState == null) 
	{
		req.readyState = 1;
		req.addEventListener("load", 
		function() 
		{
			req.readyState = 4;
			if(typeof req.onreadystatechange == "function")
			req.onreadystatechange();
		},
		false);
	}
	return req;
}

RESTClient._toBase64 = function(input)
{
	var keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
	var output = "";
	var chr1, chr2, chr3;
	var enc1, enc2, enc3, enc4;
	var i = 0;

	do {
		chr1 = input.charCodeAt(i++);
		chr2 = input.charCodeAt(i++);
		chr3 = input.charCodeAt(i++);

		enc1 = chr1 >> 2;
		enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
		enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
		enc4 = chr3 & 63;

		if (isNaN(chr2)) {
			enc3 = enc4 = 64;
		} else if (isNaN(chr3)) {
			enc4 = 64;
		}

		output = output + keyStr.charAt(enc1) + keyStr.charAt(enc2) +
		keyStr.charAt(enc3) + keyStr.charAt(enc4);
	} while (i < input.length);

	return output;
}

function ObmRESTClient() {
	this.wrappedJSObject = this;
	gLogger = Cc["@obm.org/calendar/logger;1"].createInstance(Components.interfaces.obmILogger);
	writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Components.interfaces.obmILoggerWriter);
	gLogger.setLevelWriter(gLogger.LOAD_LEVEL, writer);
	gLogger.setHeader("RESTClient.js: ");
}

ObmRESTClient.prototype = {
	//
	// nsISupports interface
	//
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	newRESTClientParameters: function() {
		return new RESTClientParameters();
	},
	invoke: function(url, method, para, async, callback, data) {
		RESTClient.invoke(url, method, para, async, callback, data);
	}
};

// module

var ObmRESTClientModule = {

	mCID: Components.ID("{00c18d10-bae3-11dd-ad8b-0800200c9a66}"),
	mContractID: "@obm.org/restclient;1",

	registerSelf: function (compMgr, fileSpec, location, type) {
		dump("registerSelf:@obm.org/restclient;1"+"\n");
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(this.mCID,
		"REST Client",
		this.mContractID,
		fileSpec,
		location,
		type);
	},

	getClassObject: function (compMgr, cid, iid) {
		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}

		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}

		return this.mFactory;
	},

	mFactory: {
		createInstance: function (outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new ObmRESTClient()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return ObmRESTClientModule;
}
