/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

function ObmRRemoteAPI() {
	this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
	var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
	this.stringBundle = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService)
						.createBundle("chrome://obm-extension/locale/calendar/errors_warnings.properties");
	this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
	this.logger.setHeader("ObmRRemoteAPIService.js: ");
};

ObmRRemoteAPI.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIRRemoteAPI)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	/*
	 * obmIRRemoteAPI
	 */

	createTaskProxy: function() {
		return Cc["@obm.org/obm-sync-rest/taskproxy;1"].createInstance(Ci.obmIRTaskProxy);
	},
	createAddressBookProxy: function() {
		return Cc["@obm.org/obm-sync-rest/addressbookproxy;1"].createInstance(Ci.obmIRAddressBookProxy);
	},
	createCalendarProxy: function() {
		this.logger.debug("createCalendarProxy()", false, false);
		var PortUrl = '';
		try {

			PortUrl = UtilsOBM.getCharPref("extensions.obm.server"); // + "/Calendar";
		} catch(e) {
			var error = "NO_SERVER_INFO";
			this.logger.error(error, true, false);
			throw (error);
		}

		var calProxy = Cc["@obm.org/obm-sync-rest/calendarproxy;1"].createInstance(Ci.obmIRCalendarProxy);
		calProxy.PortUrl = PortUrl;
		return calProxy;
	},
	createLoginProxy: function() {
		return Cc["@obm.org/obm-sync-rest/loginproxy;1"].createInstance(Ci.obmIRLoginProxy);
	}
};

// module

var ObmRRemoteAPIModule = {
	mCID: Components.ID("{f4a5a493-4c2c-4919-97bf-95059e4415ab}"),
	mContractID: "@obm.org/obm-sync-rest/service;1",
	registerSelf: function (compMgr, fileSpec, location, type) {
		dump("registerSelf:@obm.org/obm-sync-rest/service;1"+"\n");
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(
			this.mCID,
			"OBM RemoteAPI Service",
			this.mContractID,
			fileSpec,
			location,
			type
		);
	},
	getClassObject: function (compMgr, cid, iid) {

		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}

		// Load UtilsOBM.js if needed
		if (!this._utilsLoaded) {
			var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader);
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/UtilsOBM.js");
			this._utilsLoaded = true;
		}
		return this.mFactory;
	},
	mFactory: {
		createInstance: function (outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new ObmRRemoteAPI()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return ObmRRemoteAPIModule;
}

