/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - 2009-2011 Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */


function obmVCardHandler() {
	this.logger = Components.classes["@obm.org/calendar/logger;1"]
			.createInstance(Components.interfaces.obmILogger);
	var writer = Components.classes["@obm.org/calendar/writer;1?type=unixfile"]
				.getService(Components.interfaces.obmILoggerWriter);
	this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
	this.logger.setHeader("obmVCardHandler:");
}

obmVCardHandler.prototype = {
    QueryInterface: function QI(aIID) {
        if (!aIID.equals(Components.interfaces.nsISupports) &&
            !aIID.equals(Components.interfaces.obmIVCardHandler))
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    },
	getCurrentVCard : function(aMsgWindow, aDbView, aCallback) {

		var msgURI = null;
		if (aDbView) {
			msgURI = aDbView.URIForFirstSelectedMessage ;
		}
	  
		if (msgURI == null){
			if (aCallback)
				aCallback.onResponse(null, "");
			return;
		}
		//this.logger.debug("getCalendarURIFromVCard : msgURI="+msgURI,false,false);

		// let's retrieve the whole mail (its source text)
		// the streamlistener will extract the vcard
		var messenger = Components.classes["@mozilla.org/messenger;1"]
								.createInstance (Components.interfaces.nsIMessenger);
		var msgService = messenger.messageServiceFromURI (msgURI);
		this._streamListener._logger = this.logger;
		this._streamListener._callback = aCallback;
		msgService.CopyMessage (msgURI, this._streamListener, false, null, aMsgWindow, {});
	},

	/**
	 * the stream listener called during the retrieval of the mail
	 */
	_streamListener : {
		_content: "" ,
		_logger: null,
		_callback: null,
		onDataAvailable: function (request, context, inputStream, offset, count) {
			//this._logger.debug("~~~~~~ onDataAvailable ("+count+")", false, false);
			try {
				var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
									.createInstance (Components.interfaces.nsIScriptableInputStream);
				sis.init(inputStream) ;
				this._content += sis.read(count) ;
			}
			catch ( ex ) { }
		} ,
		onStartRequest: function (request , context){
			this._content = "" ;
		},
		onStopRequest: function (aRequest, aContext, aStatusCode){
			//this._logger.debug("~~~~~~ onStopRequest", false, false);
			
			// we now have the mail
			this._content = this._content.replace(/\r/g, "");
			
			// we extract the header to know the content type of the mail
			var posendHeader = this._content.indexOf("\n\n") ;
			var header = this._content.substr(0, posendHeader + 1); // keep a \n
			var headersParser = Components.classes["@mozilla.org/messenger/mimeheaders;1"]
									.createInstance(Components.interfaces.nsIMimeHeaders);
			headersParser.initialize(header, header.length) ;

			var contentType = headersParser.extractHeader ("Content-Type", false);
			var vcard = null;
			var obmCalendarURI = "";
			if (contentType && contentType.indexOf('multipart/mixed') == 0) {
				// the mail contains attachments
				//var m = /boundary="([^"]+)"/gi.exec(contentType); // it gives random results !??  FIXME: use RegExp instead
				//if (m != null) {
				var bs = contentType.indexOf('boundary="');
				var be = contentType.indexOf('"', bs+10);
				if (bs != -1 && be != -1) {
					var boundary = contentType.substr(bs+10, be-bs-10);
					//this._logger.debug(' boundary="'+boundary+'"',false,false);
					var body = this._content.substr(posendHeader+2);
					var attachments = body.split(boundary);
					for (var i=0; i < attachments.length; i++) {
						if (attachments[i].search(/^\s*Content-type\s*:\s*text\/x-vcard;/i) == 0) {
							this._logger.debug("~~~~~~    attachement "+i+" is a vcard", false, false);
							var svcard = attachments[i];
							var svcardposheader = svcard.indexOf("\n\n");
							var svcardheader = svcard.substr(0,svcardposheader);
							var svcard = svcard.substr(svcardposheader+2,  svcard.length - svcardposheader - 4); // remove the 2 last char which are "--"

							if (svcardheader.search(/\s*Content-transfer-encoding\s*:\s*base64/i) != -1) {
								// we have to decode from base64
								var wWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
										.getService(Components.interfaces.nsIWindowWatcher);
								if (wWatcher.activeWindow) {
									var win = wWatcher.activeWindow.QueryInterface(Components.interfaces.nsIDOMWindowInternal);
									svcard = win.atob(svcard.replace(/\n/g, ""));
								}
							}
							var start = svcard.search(/begin:vcard/i);
							var end = svcard.search(/end:vcard/i);
							if (start != -1 && end != -1) {
								var obmad = Components.classes['@obm.org/addressbooksmanager;1'].getService(Components.interfaces.obmIAddressBooksManager);
								vcard = obmad.escapedVCardToAbCard(encodeURIComponent(svcard.substr(start, end+9-start)));

								var vcardcontent = svcard.substr(start+11, end - start-11);

								// we won't use nsIAbCard::getProperty, TB seems to ignore the CALURI field during
								// the parsing of a vcard, so let's try to retrieve by ourselves
								var caluri = (new RegExp("\\nCALURI:\\s*([^\\s]*)","gi")).exec(vcardcontent);
								if (caluri) {
									obmCalendarURI = caluri[1];
									// store the CALURI into a custom1 field, because there is no
									// CALURI field support in TB2 and TB3
									if(Components.classes["@mozilla.org/abmanager;1"]){
										vcard.setProperty("Custom1", "X-OBM-CALURI:"+obmCalendarURI);
									}
									else {
										vcard.custom1 = "X-OBM-CALURI:"+obmCalendarURI;
									}
								}
								else
									this._logger.debug('CALURI not found',false,false);
							}
							else
								this._logger.debug('begin/end vcard not found',false,false);
						}
					}
				}
				//else
				//	this._logger.debug(' no boundary',false,false);
			}
			//else
			//	this._logger.debug(' no multipart',false,false);
			if (this._callback)
				this._callback.onResponse(vcard, obmCalendarURI);
		}
	}
};

var obmVCardHandlerModule = {
    mCID: Components.ID("{431dd1bc-6d57-42f1-b08f-18c928dd5169}"),
    mContractID: "@obm.org/vcard-handler;1",

    registerSelf: function (aCompMgr, aFileSpec, aLocation, aType) {
        var compMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(this.mCID,
                                        "OBM text/x-vcard handler",
                                        this.mContractID,
                                        aFileSpec,
                                        aLocation,
                                        aType);
    },

    getClassObject: function (aCompMgr, aCid, aIid) {
        if (!aCid.equals(this.mCID)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        if (!aIid.equals(Components.interfaces.nsIFactory)) {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
        return this.myFactory;
    },

    myFactory: {
        createInstance: function (aOuter, aIid) {
            if (aOuter != null) {
                throw Components.results.NS_ERROR_NO_AGGREGATION;
            }
			return (new obmVCardHandler()).QueryInterface(aIid);
        }
    },

    canUnload: function CU(aCompMgr) {
        return true;
    }
};

function NSGetModule(compMgr, fileSpec) {
    return obmVCardHandlerModule;
}
