/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

function StartObserver() {
	if (this.observerAdded) {
	  return;
	}

	var observerService = Components.classes["@mozilla.org/observer-service;1"]
								  .getService(Components.interfaces.nsIObserverService);
	observerService.addObserver(this, "final-ui-startup", false);
	this.observerAdded = true;
}

StartObserver.prototype = {

  //
  // nsISupports interface
  //
  QueryInterface: function(aIID) {
	if (!aIID.equals(Components.interfaces.nsISupports) && !aIID.equals(Components.interfaces.nsIObserver)) {
	  throw Components.results.NS_ERROR_NO_INTERFACE;
	}
	return this;
  },

  //
  // nsIObserver interface
  //
  observe: function(aSubject, aTopic, aData) {
	if (aTopic != "final-ui-startup") {
	  return;
	}
	
	if (!this._utilsLoaded) {
	  var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader);
	  scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/UtilsOBM.js");
	  try {
		scriptLoader.loadSubScript("chrome://calendar/content/calUtils.js");
	  } catch (e) {
		//
	  }
	  this._utilsLoaded = true;
	}

	  var mListener = {
		QueryInterface: function(aIID) {
		  if (!aIID.equals(Components.interfaces.nsISupports) && !aIID.equals(Components.interfaces.nsIConsoleListener)) {
			throw Components.results.NS_ERROR_NO_INTERFACE;
		  }
		  return this;
		},
		observe: function(msg) {
		  var str = msg.message;
		  this.logger.debug(str, false, false);
		}
	  };

	  //everything logged in JS console is write in log file
	  mListener.logger = Components.classes["@obm.org/calendar/logger;1"].createInstance(Components.interfaces.obmILogger);
	  var writer = Components.classes["@obm.org/calendar/writer;1?type=unixfile"].getService(Components.interfaces.obmILoggerWriter);
	  mListener.logger.setLevelWriter(mListener.logger.LOAD_LEVEL, writer);
	  mListener.logger.setHeader("JS console: ");

	  var Console = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
	  Console.registerListener(mListener);
	  
	  var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
	  timer.initWithCallback({ notify: function(timer) { UtilsOBM.checkLightningUpgrade(); } },
			1000, timer.TYPE_ONE_SHOT);
		this.timer = timer;
		
	  //sync on start
	  if (UtilsOBM.getBoolPref("extensions.obm.firstsync", true)) {
		dump("first sync on start canceled\n");
		UtilsOBM.setBoolPref("extensions.obm.firstsync", false);
		return;
	  }
	  var online = UtilsOBM.isOnline();
	  var pref = UtilsOBM.getBoolPref("extensions.obm.syncOnStart", true);
	  if (online && pref) {
		var username = {};
		var password = {};
		if (!UtilsOBM.getOBMCredentials(username, password) || username.value == "" || password.value == "" || password.value == "notConfiguredPassword" ) {
		  //nothing
		} else {
		  var wWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].getService(Ci.nsIWindowWatcher);
		  var params = Cc["@mozilla.org/supports-array;1"].createInstance(Ci.nsISupportsArray);
		  wWatcher.openWindow(null, "chrome://obm-extension/content/calendar/sync-dialog.xul", "OBM_SYNCHRO", "modal,centerscreen,chrome,resizable=no,dialog", params);
		}
	  }
  }
};

/****
 **** module registration
 ****/

var startObserverModule = {

  mCID: Components.ID("{16896900-4776-11dd-ae16-0800200c9a66}"),
  mContractID: "@aliasource.fr/start-observer;1",

  registerSelf: function(compMgr, fileSpec, location, type) {
	if (!this.registered) {
	  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
	  compMgr.registerFactoryLocation(this.mCID, "OBM Connector start observer", this.mContractID, fileSpec, location, type);

	  var catMgr = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);

	  catMgr.addCategoryEntry("app-startup", this.mContractID, this.mContractID, true, true);

	  this.registered = true;
	}
  },

  getClassObject: function(compMgr, cid, iid) {
	if (!cid.equals(this.mCID)) throw Components.results.NS_ERROR_NO_INTERFACE;

	if (!iid.equals(Components.interfaces.nsIFactory)) throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

	return this.mFactory;
  },

  mFactory: {
	createInstance: function(outer, iid) {
	  if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
	  return (new StartObserver()).QueryInterface(iid);
	}
  },

  canUnload: function(compMgr) {
	return true;
  }
};

function NSGetModule(compMgr, fileSpec) {
  return startObserverModule;
}